@@ -210,8 +210,8 @@ if ($opt->{reorder}) {
     use List::Util qw(first);
     for my $i (1 .. $#data) {
 	my $match =
-	    first { $data[$i]->{kana} =~ $data[$_]->{regex} } 0 .. $i - 1
-	    or next;
+	    (first { $data[$i]->{kana} =~ $data[$_]->{regex} } 0 .. $i - 1)
+	    // next;
 	splice @data, $match, 0 => splice @data, $i, 1;
     }
 }
